/**********************************************************************
**  uart_loop.c
**	Loopback characters via UART
**	Version 1.0
**
**  Copyright (C) 2017 H.Poetzl
**
**	This program is free software: you can redistribute it and/or
**	modify it under the terms of the GNU General Public License
**	as published by the Free Software Foundation, either version
**	2 of the License, or (at your option) any later version.
**
**********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <time.h>

#include <fcntl.h>
#include <stropts.h>
#include <linux/termios.h>



int	main(int argc, char *argv[])
{
	char *dev = "/dev/ttyPS0";
	unsigned long baud = 10000000;
	struct termios2 tio;
	char buf[1024];
	int fd, res;

	if (argc > 1) {
	    dev = argv[1];
	    if (argc > 2)
		baud = strtol(argv[2], NULL, 0);
	}

	fd = open(dev, O_RDWR);
	if (fd == -1) {
	    fprintf(stderr,
		"error opening >%s<.\n%s\n",
		dev, strerror(errno));
	    exit(1);
	}

	res = ioctl(fd, TCGETS2, &tio);
	if (res == -1) {
	    fprintf(stderr,
		"ioctl(TCGETS2): %s\n", strerror(errno));
	    exit(2);
	}

	tio.c_cflag &= ~PARENB;
	tio.c_cflag &= ~CSTOPB;
	tio.c_cflag &= ~CSIZE;
	tio.c_cflag &= ~CBAUD;;
	tio.c_cflag |= CS8;
	tio.c_cflag |= CLOCAL;
	tio.c_cflag |= BOTHER;

	tio.c_iflag = ICANON;
	tio.c_oflag &= ~OPOST;

	tio.c_ispeed = baud;
	tio.c_ospeed = baud;

	tio.c_cc[VTIME] = 10;
	tio.c_cc[VMIN] = 0;

	res = ioctl(fd, TCSETS2, &tio);
	if (res == -1) {
	    fprintf(stderr,
		"ioctl(TCSETS2): %s\n", strerror(errno));
	    exit(4);
	}

	res = ioctl(fd, TCFLSH, TCIOFLUSH);
	if (res == -1) {
	    fprintf(stderr,
		"ioctl(TCFLSH): %s\n", strerror(errno));
	    exit(4);
	}

	while (1) {
	    struct timespec now;
	    unsigned len;

	    len = read(fd, buf, sizeof(buf));
	    res = clock_gettime(CLOCK_REALTIME, &now);
	    if (res == -1) {
		fprintf(stderr,
		    "clock_gettime: %s\n", strerror(errno));
	    }
	    if (len > 0) {
		res = write(fd, buf, len);
		if (res == -1) {
		    fprintf(stderr,
			"write: %s\n", strerror(errno));
		} else if (res < len) {
		    fprintf(stderr,
			"short write: %u/%u\n", res, len);
		}
	    } else if (len == -1) {
		fprintf(stderr,
		    "read: %s\n", strerror(errno));
	    }

	    printf("[%lu.%09lu]\t%u\t%*.*s\n", 
		now.tv_sec, now.tv_nsec, len, len, len, buf);
	}

	close(fd);

	exit(0);
}


